from flask import Flask, render_template, jsonify
import requests
import datetime

app = Flask(__name__)

# Set the base URLs for the API
BASE_URL = "YOUR_TENANT_URL" # replace with your tenant URL i.e "https://mycompany.signalsresearch.revvitycloud.com/"
BASE_API_URL = f"{BASE_URL}api/rest/v1.0/"

HEADERS = {
    "accept": "application/vnd.api+json",
    "x-api-key": "YOUR_API_KEY", # replace with API key generated from System Settings
    "Content-Type": "application/vnd.api+json"
}

def fetch_all_users():
    """Fetches all users from the API, handling pagination automatically."""
    users_data = []
    included_roles = []
    PAGE_LIMIT = 100
    user_endpoint = f"{BASE_API_URL}users?enabled=true&page[offset]=0&page[limit]={PAGE_LIMIT}"
    
    while True:
        response = requests.get(user_endpoint, headers=HEADERS)
        if response.status_code != 200:
            print("Error fetching data:", response.status_code)
            return None
        
        data = response.json()
        users_data.extend(data.get("data", []))
        
        # Store role data separately to resolve user roles later
        if "included" in data:
            included_roles.extend(data["included"])  # Ensure we gather all roles
        
        # Check for pagination and continue fetching if needed
        next_link = data.get("links", {}).get("next")
        if not next_link:
            break  # No more pages to fetch
        
        user_endpoint = next_link
    
    return {"data": users_data, "included": included_roles}

def fetch_license_usage():
    """
    Fetches license usage details from the API.
    Processes the JSON response to build a summary of total, used, and remaining licenses.
    """
    response = requests.get(BASE_API_URL + "users/licenses", headers=HEADERS)
    if response.status_code != 200:
        print("Error fetching license data:", response.status_code)
        return None
    
    data = response.json()
    licenses = data.get("data", {}).get("attributes", {}).get("licenses", [])
    
    license_summary = []
    for license in licenses:
        total = license["totalLicense"]
        used = license["totalLicenseConsumed"]
        remaining = total - used
        license_summary.append({
            "License Name": license["name"],
            "Total Licenses": total,
            "Used Licenses": used,
            "Remaining Licenses": remaining
        })
    
    return license_summary

def fetch_entity_creation_total():
    """Fetches number of entities were created for this entityType in the last 30 days from the API."""
    
    # Get the current time in UTC
    now = datetime.datetime.now(datetime.timezone.utc)
    # Calculate the time 30 days ago
    thirty_days_ago = now - datetime.timedelta(days=30)
    
    # Format both dates to match the Signals expected date format
    from_date = thirty_days_ago.strftime("%Y-%m-%dT%H:%M:%S.000Z")
    to_date = now.strftime("%Y-%m-%dT%H:%M:%S.000Z")

    payload = {
        "query": {
            "$and": [
                {
                    "$in": {
                        "field": "type",
                          "values": [
                            "experiment",
                            "sample",
                            "task"
                          ]
                    }
                },
                {
                    "$range": {
                        "field": "createdAt",
                        "as": "date",
                        "from": from_date,
                        "to": to_date
                    }
                },
                {
                    "$match": {
                      "field": "isTemplate",
                      "value": False
                    }
                }
            ]
        },
        "field": "type"
    }

    response = requests.post(BASE_API_URL + "entities/search/terms?source=SN", headers=HEADERS, json=payload)
    
    if response.status_code != 200:
        print("Error fetching entity data:", response.status_code)
        return None
    
    data = response.json()

    return [{"Entity Type": entry["attributes"]["term"], "Total Created": entry["attributes"]["count"]} for entry in data["data"]]

def parse_users(data):
    """
    Parses raw user data and extracts key attributes.
    Also maps user roles using the included roles data.
    """
    users = []
    role_mapping = {r["id"]: r["attributes"]["name"] for r in data.get("included", []) if r["type"] == "role"}
    now = datetime.datetime.utcnow()
    
    for user in data["data"]:
        attributes = user["attributes"]
        last_login = attributes.get("lastLoginAt")
        # Convert login timestamp to a datetime object
        last_login_date = datetime.datetime.strptime(last_login, "%Y-%m-%dT%H:%M:%S.%fZ") if last_login else None
        days_since_login = (now - last_login_date).days if last_login_date else None
        
        roles = [role_mapping.get(role["id"], "Unknown") for role in user["relationships"].get("roles", {}).get("data", [])]
        licenses = [license["name"] for license in attributes.get("licenses", [])]
        
        users.append({
            "User ID": attributes.get("userId"),
            "First Name": attributes.get("firstName"),
            "Last Name": attributes.get("lastName"),
            "Email": attributes.get("email"),
            "Last Login": last_login_date.strftime("%Y-%m-%d") if last_login_date else "Never",
            "Days Since Login": days_since_login if last_login_date else "Never",
            "Licenses": ", ".join(licenses),
            "Roles": ", ".join(roles)
        })
    
    return users

def categorize_users(users):
    """
    Divides users into three categories:
    - Never Logged In
    - Inactive (30+ days)
    - Active (within 30 days)
    """
    never_logged_in = [u for u in users if u["Last Login"] == "Never"]
    over_30_days = [u for u in users if isinstance(u["Days Since Login"], int) and u["Days Since Login"] > 30]
    active_users = [u for u in users if isinstance(u["Days Since Login"], int) and u["Days Since Login"] <= 30]
    return never_logged_in, over_30_days, active_users

def get_data():
    """
    Fetches and processes all required data.
    The 'enrich' flag controls whether to add experiment details
    to active and inactive users.
    """
    all_data = fetch_all_users()
    if not all_data:
        return None, None, None, None
    users = parse_users(all_data)
    never_logged_in, over_30_days, active_users = categorize_users(users)
    license_summary = fetch_license_usage()
    entity_creation_summary = fetch_entity_creation_total()
    return never_logged_in, over_30_days, active_users, license_summary, entity_creation_summary

"""This binds the below function to the default route of our flask server"""
@app.route("/")
def index():
    """
    The main route that renders the dashboard.
    """
    never_logged_in, inactive, active, license_summary, entity_creation_summary = get_data()
    total_users = len(never_logged_in) + len(inactive) + len(active)
    summary = {
        "Total Users": total_users,
        "Active (within 30 days)": len(active),
        "Inactive (30+ days)": len(inactive),
        "Never Logged In": len(never_logged_in)
    }
    return render_template("webapp.html",
                           summary=summary,
                           license_summary=license_summary,
                           entity_creation_summary=entity_creation_summary,
                           never_logged_in=never_logged_in,
                           inactive=inactive,
                           active=active)

if __name__ == "__main__":
    app.run(debug=True)
